import os
from wiki_as_base import WikitextAsData
# from wiki_as_base import parse_sections
# from wiki_as_base import parse_tables

test_dir = os.path.dirname(os.path.realpath(__file__))


# def test_internals_001():
#     source_wikitext = test_dir + "/data/multiple.wiki.txt"
#     # target_zipfile = test_dir + "/temp/chatbotpor.zip"

#     with open(source_wikitext, "r") as _file:
#         wikitext = _file.read()

#     print(parse_sections(wikitext))
#     # assert False


# def test_internals_002():
#     source_wikitext = test_dir + "/data/multiple.wiki.txt"
#     # target_zipfile = test_dir + "/temp/chatbotpor.zip"

#     with open(source_wikitext, "r") as _file:
#         wikitext = _file.read()

#     print(parse_sections(wikitext))
#     # assert False
